unit rODAllergy;

{$O-}

interface

uses SysUtils, Classes, ORNet, ORFn, rCore, uCore, TRPCB, dialogs ;

type
  TAllergyRec = record
    Changed:             boolean;
    IEN:                 Integer;
    CausativeAgent:      string;
    NatureOfReaction:    string;
    SignsSymptoms:       TStringList;
    Originator:          int64;
    OriginatorName:      string;
    Originated:          TFMDateTime;
    SignOff:             boolean;
    Comments:            TStringList;
    IDBandMarked:        TStringList;
    ChartMarked:         TStringList;
    Verifier:            int64;
    VerifierName:        string;
    Verified:            boolean;
    VerifiedDateTime:    TFMDateTime;
    EnteredInError:      boolean;
    DateEnteredInError:  TFMDateTime;
    UserEnteringInError: int64;
    Observed_Historical: string;
    Observations:        TStringList;
    ReactionDate:        TFMDateTime;
    Severity:            string;
    NoKnownAllergies:    Boolean;
    NewComments:         TStringList;
  end;

function SearchForAllergies(StringToMatch: string): TStrings;
function SubsetofSymptoms(const StartFrom: string; Direction: Integer): TStrings;
function ODForAllergies: TStrings;
function GetCWADInfo(const DFN: string): string;
function OldSaveAllergy(EditRec: TAllergyRec): string;
function NewSaveAllergy(EditRec: TAllergyRec): string;
function LoadAllergyForEdit(AllergyIEN: integer): TAllergyRec;

implementation

function ODForAllergies: TStrings;
begin
  CallV('ORWDAL32 DEF',[nil]);
  Result := RPCBrokerV.Results;
end;

function SearchForAllergies(StringToMatch: string): TStrings;
begin
  CallV('ORWDAL32 ALLERGY MATCH',[StringToMatch]);
  Result := RPCBrokerV.Results;
end;

function SubsetofSymptoms(const StartFrom: string; Direction: Integer): TStrings;
begin
  Callv('ORWDAL32 SYMPTOMS',[StartFrom, Direction]);
  Result := RPCBrokerV.Results;
end;

function GetCWADInfo(const DFN: string): string;
begin
  Result := sCallV('ORWPT CWAD',[DFN]);
end;

function LoadAllergyForEdit(AllergyIEN: integer): TAllergyRec;
var
  Dest: TStringList;
  EditRec: TAllergyRec;
begin
  Dest := TStringList.Create;
  try
    tCallV(Dest, 'ORWDAL32 LOAD FOR EDIT', [AllergyIEN]) ;
    with EditRec do
      begin
        Changed             := False;
        IEN                 := AllergyIEN;
        CausativeAgent      := ExtractDefault(Dest, 'CAUSATIVE AGENT');
        NatureOfReaction    := ExtractDefault(Dest, 'NATURE OF REACTION');
        SignsSymptoms       := TStringList.Create;
        ExtractItems(SignsSymptoms, Dest, 'SIGN/SYMPTOMS');
        MixedCaseByPiece(SignsSymptoms, U, 4);
        Originator          := StrToInt64Def(Piece(ExtractDefault(Dest, 'ORIGINATOR'), U, 1), 0);
        OriginatorName      := Piece(ExtractDefault(Dest, 'ORIGINATOR'), U, 2);
        Originated          := StrToFMDateTime(ExtractDefault(Dest, 'ORIGINATED'));
        SignOff             := ExtractDefault(Dest, 'SIGN OFF') = 'YES';
        Comments            := TStringList.Create;
        ExtractText(Comments, Dest, 'COMMENTS');
        IDBandMarked        := TStringList.Create;
        ExtractItems(IDBandMarked, Dest, 'ID BAND MARKED');
        ChartMarked        := TStringList.Create;
        ExtractItems(ChartMarked, Dest, 'CHART MARKED');
        Verifier            := StrToInt64Def(Piece(ExtractDefault(Dest, 'VERIFIER'), U, 1), 0);
        VerifierName        := Piece(ExtractDefault(Dest, 'VERIFIER'), U, 2);
        Verified            := ExtractDefault(Dest, 'VERIFIED') = 'YES';
        if Verified then
          VerifiedDateTime  := StrToFMDateTime(Piece(ExtractDefault(Dest, 'VERIFIED'), U, 2));
        EnteredInError      := Piece(ExtractDefault(Dest, 'ENTERED IN ERROR'), U, 1) = 'YES';
        DateEnteredInError  := StrToFloatDef(Piece(ExtractDefault(Dest, 'ENTERED IN ERROR'), U, 2), 0);
        UserEnteringInError := StrToInt64Def(Piece(ExtractDefault(Dest, 'ENTERED IN ERROR'), U, 3), 0);
        Observed_Historical := ExtractDefault(Dest, 'OBS/HIST');
        Observations        := TStringList.Create;
        ExtractText(Observations, Dest, 'OBSERVATIONS');
        //ReactionDate        := StrToFMDateTime(Piece(ExtractDefault(Dest, 'REACTDT'), U, 3));
        //Severity            := Piece(ExtractDefault(Dest, 'SEVERITY'), U, 3);
        NoKnownAllergies    := (StrToIntDef(Piece(ExtractDefault(Dest, 'NKA'), U, 3), 0) > 0);
        NewComments         := TStringList.Create;
      end;
    Result := EditRec;
  finally
    Dest.Free;
  end;
end;

{ TODO -oRich V. -cART/Allergy : Before Diane's new proposal. }
function OldSaveAllergy(EditRec: TAllergyRec): string;
var
  i: integer;
begin
  with RPCBrokerV, EditRec do
    begin
      ClearParameters := True;
      RemoteProcedure := 'ORWDAL32 SAVE ALLERGY';
      Param[0].PType := literal;
      Param[0].Value := IntToStr(IEN);
      Param[1].PType := literal;
      Param[1].Value := Patient.DFN;
      Param[2].PType := list;
      with Param[2] do
        begin
          if NoKnownAllergies then
            Mult['1']  := 'GMRANKA^'  + BOOLCHAR[NoKnownAllergies];
          if CausativeAgent <> '' then
            Mult['2']  := 'GMRAGNT^'  + CausativeAgent;
          if NatureOfReaction <> '' then
            Mult['3']  := 'GMRANATR^' + NatureOfReaction ;
          if Originator > 0 then
            Mult['4']  := 'GMRAORIG^'  + IntToStr(Originator);
          if Originated > 0 then
            Mult['5']  := 'GMRAORDT^'  + FloatToStr(Originated);
          with SignsSymptoms do if Count > 0 then
            begin
              Mult['6'] := 'GMRASYMP^' + IntToStr(Count);
              for i := 0 to Count - 1 do
                Mult['6,' + IntToStr(i+1)] := Piece(Strings[i], U, 1) + U + Piece(Strings[i], U, 3);
            end;
{ TODO -oRich V. -cART/Allergy : Alternative array subscripting format ?? }
(*          with SignsSymptoms do if Count > 0 then
            begin
              Mult['"GMRASYMP",0'] := IntToStr(Count);
              for i := 0 to Count - 1 do
                Mult['"GMRASYMP",' + IntToStr(i+1)] := Piece(Strings[i], U, 1) + U + Piece(Strings[i], U, 3);
            end;*)
          if Verified then
            Mult['7']  := 'GMRAVER^'  + BOOLCHAR[Verified];
          if Verifier > 0 then
            Mult['8']  := 'GMRAVERF^'  + IntToStr(Verifier);
          if VerifiedDateTime > 0 then
            Mult['9']  := 'GMRAVERD^'  + FloatToStr(VerifiedDateTime);
          if EnteredInError then
            Mult['10']  := 'GMRAERR^'  + BOOLCHAR[EnteredInError];
          if SignOff then
            Mult['11']  := 'GMRASGN^'  + BOOLCHAR[SignOff];
          with ChartMarked do if Count > 0 then
            begin
              Mult['12'] := 'GMRACHT^' + IntToStr(Count);
              for i := 0 to Count - 1 do
                Mult['12,' + IntToStr(i+1)] := Strings[i];
            end;
          with IDBandMarked do if Count > 0 then
            begin
              Mult['13'] := 'GMRAIDBN^' + IntToStr(Count);
              for i := 0 to Count - 1 do
                Mult['13,' + IntToStr(i+1)] := Strings[i];
            end;
          if Length(Observed_Historical) > 0 then
            Mult['14']  := 'GMRAOBHX^'   + Observed_Historical;
          if ReactionDate > 0 then
            Mult['15']  := 'GMRARDT^'  + FloatToStr(ReactionDate);
          if Length(Severity) > 0 then
            Mult['16']  := 'GMRASEVR^'  + Severity;
          with NewComments do if Count > 0 then
            begin
              Mult['17'] := 'GMRACMTS^' + IntToStr(Count);
              for i := 0 to Count - 1 do
                Mult['17,' + IntToStr(i+1)] := Strings[i];
            end;
        end;
      CallBroker;
      Result := Results[0];
    end;
end;

{ TODO -oRich V. -cART/Allergy : After Diane's new proposal. }
function NewSaveAllergy(EditRec: TAllergyRec): string;
const
  NO_YES: array[Boolean] of string = ('NO', 'YES');
var
  i: integer;
  tmpList: TStringList;
begin
  tmpList := TStringList.Create;
  try
    with EditRec, tmpList do
      begin
        Add('~CAUSATIVE AGENT');
        Add('d' + CausativeAgent);
        Add('~NATURE OF REACTION');
        Add('d' + NatureOfReaction);
        Add('~SIGN/SYMPTOMS');
        with SignsSymptoms do for i := 0 to Count - 1 do
          tmpList.Add('i' + Piece(SignsSymptoms[i], U, 1) + U + Piece(SignsSymptoms[i], U, 3));
        Add('~ORIGINATOR');
        Add('d' + IntToStr(Originator));
        Add('~ORIGINATED');
        Add('d' + FloatToStr(Originated));
        Add('~SIGN OFF');
        Add('d' + NO_YES[SignOff]);
        Add('~ID BAND MARKED');
        with IDBandMarked do for i := 0 to Count - 1 do
          tmpList.Add('i' + IDBandMarked[i]);
        Add('~CHART MARKED');
        with ChartMarked do for i := 0 to Count - 1 do
          tmpList.Add('i' + ChartMarked[i]);
        Add('~VERIFIER');
        Add('d' + IntToStr(Verifier));
        Add('~VERIFIED');
        Add('d' + NO_YES[Verified] + U + FloatToStr(VerifiedDateTime));
        Add('~ENTERED IN ERROR');
        Add('d' + NO_YES[EnteredInError] + U + FloatToStr(DateEnteredInError) + U + IntToStr(UserEnteringInError));
{ TODO -oRich V. -cART/Allergy : How to handle Observations multiple? }
        Add('~OBS/HIST');
        Add('d' + Observed_Historical);
        Add('~SEVERITY');
        Add('d' + Severity);
        Add('~REACTION DATE/TIME');
        Add('d' + FloatToStr(ReactionDate));
{ TODO -oRich V. -cART/Allergy : How to handle Comments multiple? }
        Add('~COMMENTS');
        with NewComments do for i := 0 to Count - 1 do
          tmpList.Add('t' + NewComments[i]);
      end;

    with RPCBrokerV do
      begin
        ClearParameters := True;
        RemoteProcedure := 'ORWDAL32 SAVE ALLERGY';
        Param[0].PType := literal;
        Param[0].Value := IntToStr(EditRec.IEN);
        Param[1].PType := literal;
        Param[1].Value := Patient.DFN;
        Param[2].PType := list;
        with Param[2] do
          for i := 0 to tmpList.Count - 1 do
            Mult[IntToStr(Succ(i))] := tmpList[i];
        CallBroker;
        Result := Results[0];
      end;
  finally
    tmpList.Free;
  end;
end;

end.
